# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc4bmi/bmi.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='grpc4bmi/bmi.proto',
  package='bmi',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x12grpc4bmi/bmi.proto\x12\x03\x62mi\"\x07\n\x05\x45mpty\"(\n\x11InitializeRequest\x12\x13\n\x0b\x63onfig_file\x18\x01 \x01(\t\"(\n\x18GetComponentNameResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\"$\n\x13GetVarNamesResponse\x12\r\n\x05names\x18\x01 \x03(\t\"%\n\x14GetTimeUnitsResponse\x12\r\n\x05units\x18\x01 \x01(\t\"\'\n\x13GetTimeStepResponse\x12\x10\n\x08interval\x18\x01 \x01(\x01\"\x1f\n\x0fGetTimeResponse\x12\x0c\n\x04time\x18\x01 \x01(\x01\"\x1d\n\rGetVarRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"%\n\x12GetVarGridResponse\x12\x0f\n\x07grid_id\x18\x01 \x01(\x05\"\"\n\x12GetVarTypeResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\"&\n\x16GetVarItemSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"$\n\x13GetVarUnitsResponse\x12\r\n\x05units\x18\x01 \x01(\t\"&\n\x14GetVarNBytesResponse\x12\x0e\n\x06nbytes\x18\x01 \x01(\x03\"z\n\x16GetVarLocationResponse\x12\x36\n\x08location\x18\x01 \x01(\x0e\x32$.bmi.GetVarLocationResponse.Location\"(\n\x08Location\x12\x08\n\x04NODE\x10\x00\x12\x08\n\x04\x45\x44GE\x10\x01\x12\x08\n\x04\x46\x41\x43\x45\x10\x02\"%\n\x0fIntArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x03\x42\x02\x10\x01\"\'\n\x11\x46loatArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x02\x42\x02\x10\x01\"(\n\x12\x44oubleArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x01\x42\x02\x10\x01\"\xaa\x01\n\x10GetValueResponse\x12*\n\nvalues_int\x18\x01 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x02 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x03 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"=\n\x18GetValueAtIndicesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x07indices\x18\x02 \x03(\x03\x42\x02\x10\x01\"\xb3\x01\n\x19GetValueAtIndicesResponse\x12*\n\nvalues_int\x18\x01 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x02 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x03 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"\xb7\x01\n\x0fSetValueRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12*\n\nvalues_int\x18\x02 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x03 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x04 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"/\n\x12SetValuePtrRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03ref\x18\x02 \x01(\x03\"\xd5\x01\n\x18SetValueAtIndicesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x07indices\x18\x02 \x03(\x03\x42\x02\x10\x01\x12*\n\nvalues_int\x18\x03 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x04 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x05 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"\x1e\n\x0bGridRequest\x12\x0f\n\x07grid_id\x18\x01 \x01(\x03\"#\n\x13GetGridSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"#\n\x13GetGridRankResponse\x12\x0c\n\x04rank\x18\x01 \x01(\x03\"#\n\x13GetGridTypeResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\")\n\x14GetGridShapeResponse\x12\x11\n\x05shape\x18\x01 \x03(\x03\x42\x02\x10\x01\"-\n\x16GetGridSpacingResponse\x12\x13\n\x07spacing\x18\x01 \x03(\x01\x42\x02\x10\x01\"+\n\x15GetGridOriginResponse\x12\x12\n\x06origin\x18\x01 \x03(\x01\x42\x02\x10\x01\"0\n\x15GetGridPointsResponse\x12\x17\n\x0b\x63oordinates\x18\x01 \x03(\x01\x42\x02\x10\x01\"!\n\x10GetCountResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\"2\n\x18GetGridEdgeNodesResponse\x12\x16\n\nedge_nodes\x18\x01 \x03(\x03\x42\x02\x10\x01\"2\n\x18GetGridFaceEdgesResponse\x12\x16\n\nface_edges\x18\x01 \x03(\x03\x42\x02\x10\x01\"2\n\x18GetGridFaceNodesResponse\x12\x16\n\nface_nodes\x18\x01 \x03(\x03\x42\x02\x10\x01\"9\n\x1bGetGridNodesPerFaceResponse\x12\x1a\n\x0enodes_per_face\x18\x01 \x03(\x03\x42\x02\x10\x01\x32\x8a\x13\n\nBmiService\x12\x32\n\ninitialize\x12\x16.bmi.InitializeRequest\x1a\n.bmi.Empty\"\x00\x12\"\n\x06update\x12\n.bmi.Empty\x1a\n.bmi.Empty\"\x00\x12\x31\n\x0bupdateUntil\x12\x14.bmi.GetTimeResponse\x1a\n.bmi.Empty\"\x00\x12$\n\x08\x66inalize\x12\n.bmi.Empty\x1a\n.bmi.Empty\"\x00\x12?\n\x10getComponentName\x12\n.bmi.Empty\x1a\x1d.bmi.GetComponentNameResponse\"\x00\x12\x38\n\x11getInputItemCount\x12\n.bmi.Empty\x1a\x15.bmi.GetCountResponse\"\x00\x12\x39\n\x12getOutputItemCount\x12\n.bmi.Empty\x1a\x15.bmi.GetCountResponse\"\x00\x12:\n\x10getInputVarNames\x12\n.bmi.Empty\x1a\x18.bmi.GetVarNamesResponse\"\x00\x12;\n\x11getOutputVarNames\x12\n.bmi.Empty\x1a\x18.bmi.GetVarNamesResponse\"\x00\x12\x37\n\x0cgetTimeUnits\x12\n.bmi.Empty\x1a\x19.bmi.GetTimeUnitsResponse\"\x00\x12\x35\n\x0bgetTimeStep\x12\n.bmi.Empty\x1a\x18.bmi.GetTimeStepResponse\"\x00\x12\x34\n\x0egetCurrentTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12\x32\n\x0cgetStartTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12\x30\n\ngetEndTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12;\n\ngetVarGrid\x12\x12.bmi.GetVarRequest\x1a\x17.bmi.GetVarGridResponse\"\x00\x12;\n\ngetVarType\x12\x12.bmi.GetVarRequest\x1a\x17.bmi.GetVarTypeResponse\"\x00\x12\x43\n\x0egetVarItemSize\x12\x12.bmi.GetVarRequest\x1a\x1b.bmi.GetVarItemSizeResponse\"\x00\x12=\n\x0bgetVarUnits\x12\x12.bmi.GetVarRequest\x1a\x18.bmi.GetVarUnitsResponse\"\x00\x12?\n\x0cgetVarNBytes\x12\x12.bmi.GetVarRequest\x1a\x19.bmi.GetVarNBytesResponse\"\x00\x12\x43\n\x0egetVarLocation\x12\x12.bmi.GetVarRequest\x1a\x1b.bmi.GetVarLocationResponse\"\x00\x12\x37\n\x08getValue\x12\x12.bmi.GetVarRequest\x1a\x15.bmi.GetValueResponse\"\x00\x12T\n\x11getValueAtIndices\x12\x1d.bmi.GetValueAtIndicesRequest\x1a\x1e.bmi.GetValueAtIndicesResponse\"\x00\x12.\n\x08setValue\x12\x14.bmi.SetValueRequest\x1a\n.bmi.Empty\"\x00\x12@\n\x11setValueAtIndices\x12\x1d.bmi.SetValueAtIndicesRequest\x1a\n.bmi.Empty\"\x00\x12;\n\x0bgetGridSize\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridSizeResponse\"\x00\x12;\n\x0bgetGridType\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridTypeResponse\"\x00\x12;\n\x0bgetGridRank\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridRankResponse\"\x00\x12=\n\x0cgetGridShape\x12\x10.bmi.GridRequest\x1a\x19.bmi.GetGridShapeResponse\"\x00\x12\x41\n\x0egetGridSpacing\x12\x10.bmi.GridRequest\x1a\x1b.bmi.GetGridSpacingResponse\"\x00\x12?\n\rgetGridOrigin\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridOriginResponse\"\x00\x12:\n\x08getGridX\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12:\n\x08getGridY\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12:\n\x08getGridZ\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12=\n\x10getGridNodeCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12=\n\x10getGridEdgeCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12=\n\x10getGridFaceCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12\x45\n\x10getGridEdgeNodes\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridEdgeNodesResponse\"\x00\x12\x45\n\x10getGridFaceNodes\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridFaceNodesResponse\"\x00\x12\x45\n\x10getGridFaceEdges\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridFaceEdgesResponse\"\x00\x12K\n\x13getGridNodesPerFace\x12\x10.bmi.GridRequest\x1a .bmi.GetGridNodesPerFaceResponse\"\x00\x62\x06proto3'
)



_GETVARLOCATIONRESPONSE_LOCATION = _descriptor.EnumDescriptor(
  name='Location',
  full_name='bmi.GetVarLocationResponse.Location',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NODE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EDGE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FACE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=577,
  serialized_end=617,
)
_sym_db.RegisterEnumDescriptor(_GETVARLOCATIONRESPONSE_LOCATION)


_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='bmi.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27,
  serialized_end=34,
)


_INITIALIZEREQUEST = _descriptor.Descriptor(
  name='InitializeRequest',
  full_name='bmi.InitializeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config_file', full_name='bmi.InitializeRequest.config_file', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=76,
)


_GETCOMPONENTNAMERESPONSE = _descriptor.Descriptor(
  name='GetComponentNameResponse',
  full_name='bmi.GetComponentNameResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.GetComponentNameResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=118,
)


_GETVARNAMESRESPONSE = _descriptor.Descriptor(
  name='GetVarNamesResponse',
  full_name='bmi.GetVarNamesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='bmi.GetVarNamesResponse.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=156,
)


_GETTIMEUNITSRESPONSE = _descriptor.Descriptor(
  name='GetTimeUnitsResponse',
  full_name='bmi.GetTimeUnitsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='units', full_name='bmi.GetTimeUnitsResponse.units', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=158,
  serialized_end=195,
)


_GETTIMESTEPRESPONSE = _descriptor.Descriptor(
  name='GetTimeStepResponse',
  full_name='bmi.GetTimeStepResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interval', full_name='bmi.GetTimeStepResponse.interval', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=197,
  serialized_end=236,
)


_GETTIMERESPONSE = _descriptor.Descriptor(
  name='GetTimeResponse',
  full_name='bmi.GetTimeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='bmi.GetTimeResponse.time', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=238,
  serialized_end=269,
)


_GETVARREQUEST = _descriptor.Descriptor(
  name='GetVarRequest',
  full_name='bmi.GetVarRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.GetVarRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=271,
  serialized_end=300,
)


_GETVARGRIDRESPONSE = _descriptor.Descriptor(
  name='GetVarGridResponse',
  full_name='bmi.GetVarGridResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='grid_id', full_name='bmi.GetVarGridResponse.grid_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=302,
  serialized_end=339,
)


_GETVARTYPERESPONSE = _descriptor.Descriptor(
  name='GetVarTypeResponse',
  full_name='bmi.GetVarTypeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='bmi.GetVarTypeResponse.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=341,
  serialized_end=375,
)


_GETVARITEMSIZERESPONSE = _descriptor.Descriptor(
  name='GetVarItemSizeResponse',
  full_name='bmi.GetVarItemSizeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='bmi.GetVarItemSizeResponse.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=377,
  serialized_end=415,
)


_GETVARUNITSRESPONSE = _descriptor.Descriptor(
  name='GetVarUnitsResponse',
  full_name='bmi.GetVarUnitsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='units', full_name='bmi.GetVarUnitsResponse.units', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=417,
  serialized_end=453,
)


_GETVARNBYTESRESPONSE = _descriptor.Descriptor(
  name='GetVarNBytesResponse',
  full_name='bmi.GetVarNBytesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nbytes', full_name='bmi.GetVarNBytesResponse.nbytes', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=455,
  serialized_end=493,
)


_GETVARLOCATIONRESPONSE = _descriptor.Descriptor(
  name='GetVarLocationResponse',
  full_name='bmi.GetVarLocationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='location', full_name='bmi.GetVarLocationResponse.location', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETVARLOCATIONRESPONSE_LOCATION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=495,
  serialized_end=617,
)


_INTARRAYMESSAGE = _descriptor.Descriptor(
  name='IntArrayMessage',
  full_name='bmi.IntArrayMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='bmi.IntArrayMessage.values', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=619,
  serialized_end=656,
)


_FLOATARRAYMESSAGE = _descriptor.Descriptor(
  name='FloatArrayMessage',
  full_name='bmi.FloatArrayMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='bmi.FloatArrayMessage.values', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=658,
  serialized_end=697,
)


_DOUBLEARRAYMESSAGE = _descriptor.Descriptor(
  name='DoubleArrayMessage',
  full_name='bmi.DoubleArrayMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='bmi.DoubleArrayMessage.values', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=699,
  serialized_end=739,
)


_GETVALUERESPONSE = _descriptor.Descriptor(
  name='GetValueResponse',
  full_name='bmi.GetValueResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.GetValueResponse.values_int', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.GetValueResponse.values_float', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.GetValueResponse.values_double', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.GetValueResponse.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=742,
  serialized_end=912,
)


_GETVALUEATINDICESREQUEST = _descriptor.Descriptor(
  name='GetValueAtIndicesRequest',
  full_name='bmi.GetValueAtIndicesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.GetValueAtIndicesRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indices', full_name='bmi.GetValueAtIndicesRequest.indices', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=914,
  serialized_end=975,
)


_GETVALUEATINDICESRESPONSE = _descriptor.Descriptor(
  name='GetValueAtIndicesResponse',
  full_name='bmi.GetValueAtIndicesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.GetValueAtIndicesResponse.values_int', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.GetValueAtIndicesResponse.values_float', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.GetValueAtIndicesResponse.values_double', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.GetValueAtIndicesResponse.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=978,
  serialized_end=1157,
)


_SETVALUEREQUEST = _descriptor.Descriptor(
  name='SetValueRequest',
  full_name='bmi.SetValueRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.SetValueRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.SetValueRequest.values_int', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.SetValueRequest.values_float', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.SetValueRequest.values_double', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.SetValueRequest.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1160,
  serialized_end=1343,
)


_SETVALUEPTRREQUEST = _descriptor.Descriptor(
  name='SetValuePtrRequest',
  full_name='bmi.SetValuePtrRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.SetValuePtrRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ref', full_name='bmi.SetValuePtrRequest.ref', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1345,
  serialized_end=1392,
)


_SETVALUEATINDICESREQUEST = _descriptor.Descriptor(
  name='SetValueAtIndicesRequest',
  full_name='bmi.SetValueAtIndicesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bmi.SetValueAtIndicesRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indices', full_name='bmi.SetValueAtIndicesRequest.indices', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_int', full_name='bmi.SetValueAtIndicesRequest.values_int', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_float', full_name='bmi.SetValueAtIndicesRequest.values_float', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values_double', full_name='bmi.SetValueAtIndicesRequest.values_double', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='values', full_name='bmi.SetValueAtIndicesRequest.values',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1395,
  serialized_end=1608,
)


_GRIDREQUEST = _descriptor.Descriptor(
  name='GridRequest',
  full_name='bmi.GridRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='grid_id', full_name='bmi.GridRequest.grid_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1610,
  serialized_end=1640,
)


_GETGRIDSIZERESPONSE = _descriptor.Descriptor(
  name='GetGridSizeResponse',
  full_name='bmi.GetGridSizeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='bmi.GetGridSizeResponse.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1642,
  serialized_end=1677,
)


_GETGRIDRANKRESPONSE = _descriptor.Descriptor(
  name='GetGridRankResponse',
  full_name='bmi.GetGridRankResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rank', full_name='bmi.GetGridRankResponse.rank', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1679,
  serialized_end=1714,
)


_GETGRIDTYPERESPONSE = _descriptor.Descriptor(
  name='GetGridTypeResponse',
  full_name='bmi.GetGridTypeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='bmi.GetGridTypeResponse.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1716,
  serialized_end=1751,
)


_GETGRIDSHAPERESPONSE = _descriptor.Descriptor(
  name='GetGridShapeResponse',
  full_name='bmi.GetGridShapeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='shape', full_name='bmi.GetGridShapeResponse.shape', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1753,
  serialized_end=1794,
)


_GETGRIDSPACINGRESPONSE = _descriptor.Descriptor(
  name='GetGridSpacingResponse',
  full_name='bmi.GetGridSpacingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='spacing', full_name='bmi.GetGridSpacingResponse.spacing', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1796,
  serialized_end=1841,
)


_GETGRIDORIGINRESPONSE = _descriptor.Descriptor(
  name='GetGridOriginResponse',
  full_name='bmi.GetGridOriginResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='origin', full_name='bmi.GetGridOriginResponse.origin', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1843,
  serialized_end=1886,
)


_GETGRIDPOINTSRESPONSE = _descriptor.Descriptor(
  name='GetGridPointsResponse',
  full_name='bmi.GetGridPointsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='coordinates', full_name='bmi.GetGridPointsResponse.coordinates', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1888,
  serialized_end=1936,
)


_GETCOUNTRESPONSE = _descriptor.Descriptor(
  name='GetCountResponse',
  full_name='bmi.GetCountResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='count', full_name='bmi.GetCountResponse.count', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1938,
  serialized_end=1971,
)


_GETGRIDEDGENODESRESPONSE = _descriptor.Descriptor(
  name='GetGridEdgeNodesResponse',
  full_name='bmi.GetGridEdgeNodesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='edge_nodes', full_name='bmi.GetGridEdgeNodesResponse.edge_nodes', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1973,
  serialized_end=2023,
)


_GETGRIDFACEEDGESRESPONSE = _descriptor.Descriptor(
  name='GetGridFaceEdgesResponse',
  full_name='bmi.GetGridFaceEdgesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='face_edges', full_name='bmi.GetGridFaceEdgesResponse.face_edges', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2025,
  serialized_end=2075,
)


_GETGRIDFACENODESRESPONSE = _descriptor.Descriptor(
  name='GetGridFaceNodesResponse',
  full_name='bmi.GetGridFaceNodesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='face_nodes', full_name='bmi.GetGridFaceNodesResponse.face_nodes', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2077,
  serialized_end=2127,
)


_GETGRIDNODESPERFACERESPONSE = _descriptor.Descriptor(
  name='GetGridNodesPerFaceResponse',
  full_name='bmi.GetGridNodesPerFaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nodes_per_face', full_name='bmi.GetGridNodesPerFaceResponse.nodes_per_face', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2129,
  serialized_end=2186,
)

_GETVARLOCATIONRESPONSE.fields_by_name['location'].enum_type = _GETVARLOCATIONRESPONSE_LOCATION
_GETVARLOCATIONRESPONSE_LOCATION.containing_type = _GETVARLOCATIONRESPONSE
_GETVALUERESPONSE.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_GETVALUERESPONSE.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_GETVALUERESPONSE.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_GETVALUERESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUERESPONSE.fields_by_name['values_int'])
_GETVALUERESPONSE.fields_by_name['values_int'].containing_oneof = _GETVALUERESPONSE.oneofs_by_name['values']
_GETVALUERESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUERESPONSE.fields_by_name['values_float'])
_GETVALUERESPONSE.fields_by_name['values_float'].containing_oneof = _GETVALUERESPONSE.oneofs_by_name['values']
_GETVALUERESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUERESPONSE.fields_by_name['values_double'])
_GETVALUERESPONSE.fields_by_name['values_double'].containing_oneof = _GETVALUERESPONSE.oneofs_by_name['values']
_GETVALUEATINDICESRESPONSE.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_GETVALUEATINDICESRESPONSE.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_GETVALUEATINDICESRESPONSE.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_GETVALUEATINDICESRESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUEATINDICESRESPONSE.fields_by_name['values_int'])
_GETVALUEATINDICESRESPONSE.fields_by_name['values_int'].containing_oneof = _GETVALUEATINDICESRESPONSE.oneofs_by_name['values']
_GETVALUEATINDICESRESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUEATINDICESRESPONSE.fields_by_name['values_float'])
_GETVALUEATINDICESRESPONSE.fields_by_name['values_float'].containing_oneof = _GETVALUEATINDICESRESPONSE.oneofs_by_name['values']
_GETVALUEATINDICESRESPONSE.oneofs_by_name['values'].fields.append(
  _GETVALUEATINDICESRESPONSE.fields_by_name['values_double'])
_GETVALUEATINDICESRESPONSE.fields_by_name['values_double'].containing_oneof = _GETVALUEATINDICESRESPONSE.oneofs_by_name['values']
_SETVALUEREQUEST.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_SETVALUEREQUEST.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_SETVALUEREQUEST.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_SETVALUEREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEREQUEST.fields_by_name['values_int'])
_SETVALUEREQUEST.fields_by_name['values_int'].containing_oneof = _SETVALUEREQUEST.oneofs_by_name['values']
_SETVALUEREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEREQUEST.fields_by_name['values_float'])
_SETVALUEREQUEST.fields_by_name['values_float'].containing_oneof = _SETVALUEREQUEST.oneofs_by_name['values']
_SETVALUEREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEREQUEST.fields_by_name['values_double'])
_SETVALUEREQUEST.fields_by_name['values_double'].containing_oneof = _SETVALUEREQUEST.oneofs_by_name['values']
_SETVALUEATINDICESREQUEST.fields_by_name['values_int'].message_type = _INTARRAYMESSAGE
_SETVALUEATINDICESREQUEST.fields_by_name['values_float'].message_type = _FLOATARRAYMESSAGE
_SETVALUEATINDICESREQUEST.fields_by_name['values_double'].message_type = _DOUBLEARRAYMESSAGE
_SETVALUEATINDICESREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEATINDICESREQUEST.fields_by_name['values_int'])
_SETVALUEATINDICESREQUEST.fields_by_name['values_int'].containing_oneof = _SETVALUEATINDICESREQUEST.oneofs_by_name['values']
_SETVALUEATINDICESREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEATINDICESREQUEST.fields_by_name['values_float'])
_SETVALUEATINDICESREQUEST.fields_by_name['values_float'].containing_oneof = _SETVALUEATINDICESREQUEST.oneofs_by_name['values']
_SETVALUEATINDICESREQUEST.oneofs_by_name['values'].fields.append(
  _SETVALUEATINDICESREQUEST.fields_by_name['values_double'])
_SETVALUEATINDICESREQUEST.fields_by_name['values_double'].containing_oneof = _SETVALUEATINDICESREQUEST.oneofs_by_name['values']
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['InitializeRequest'] = _INITIALIZEREQUEST
DESCRIPTOR.message_types_by_name['GetComponentNameResponse'] = _GETCOMPONENTNAMERESPONSE
DESCRIPTOR.message_types_by_name['GetVarNamesResponse'] = _GETVARNAMESRESPONSE
DESCRIPTOR.message_types_by_name['GetTimeUnitsResponse'] = _GETTIMEUNITSRESPONSE
DESCRIPTOR.message_types_by_name['GetTimeStepResponse'] = _GETTIMESTEPRESPONSE
DESCRIPTOR.message_types_by_name['GetTimeResponse'] = _GETTIMERESPONSE
DESCRIPTOR.message_types_by_name['GetVarRequest'] = _GETVARREQUEST
DESCRIPTOR.message_types_by_name['GetVarGridResponse'] = _GETVARGRIDRESPONSE
DESCRIPTOR.message_types_by_name['GetVarTypeResponse'] = _GETVARTYPERESPONSE
DESCRIPTOR.message_types_by_name['GetVarItemSizeResponse'] = _GETVARITEMSIZERESPONSE
DESCRIPTOR.message_types_by_name['GetVarUnitsResponse'] = _GETVARUNITSRESPONSE
DESCRIPTOR.message_types_by_name['GetVarNBytesResponse'] = _GETVARNBYTESRESPONSE
DESCRIPTOR.message_types_by_name['GetVarLocationResponse'] = _GETVARLOCATIONRESPONSE
DESCRIPTOR.message_types_by_name['IntArrayMessage'] = _INTARRAYMESSAGE
DESCRIPTOR.message_types_by_name['FloatArrayMessage'] = _FLOATARRAYMESSAGE
DESCRIPTOR.message_types_by_name['DoubleArrayMessage'] = _DOUBLEARRAYMESSAGE
DESCRIPTOR.message_types_by_name['GetValueResponse'] = _GETVALUERESPONSE
DESCRIPTOR.message_types_by_name['GetValueAtIndicesRequest'] = _GETVALUEATINDICESREQUEST
DESCRIPTOR.message_types_by_name['GetValueAtIndicesResponse'] = _GETVALUEATINDICESRESPONSE
DESCRIPTOR.message_types_by_name['SetValueRequest'] = _SETVALUEREQUEST
DESCRIPTOR.message_types_by_name['SetValuePtrRequest'] = _SETVALUEPTRREQUEST
DESCRIPTOR.message_types_by_name['SetValueAtIndicesRequest'] = _SETVALUEATINDICESREQUEST
DESCRIPTOR.message_types_by_name['GridRequest'] = _GRIDREQUEST
DESCRIPTOR.message_types_by_name['GetGridSizeResponse'] = _GETGRIDSIZERESPONSE
DESCRIPTOR.message_types_by_name['GetGridRankResponse'] = _GETGRIDRANKRESPONSE
DESCRIPTOR.message_types_by_name['GetGridTypeResponse'] = _GETGRIDTYPERESPONSE
DESCRIPTOR.message_types_by_name['GetGridShapeResponse'] = _GETGRIDSHAPERESPONSE
DESCRIPTOR.message_types_by_name['GetGridSpacingResponse'] = _GETGRIDSPACINGRESPONSE
DESCRIPTOR.message_types_by_name['GetGridOriginResponse'] = _GETGRIDORIGINRESPONSE
DESCRIPTOR.message_types_by_name['GetGridPointsResponse'] = _GETGRIDPOINTSRESPONSE
DESCRIPTOR.message_types_by_name['GetCountResponse'] = _GETCOUNTRESPONSE
DESCRIPTOR.message_types_by_name['GetGridEdgeNodesResponse'] = _GETGRIDEDGENODESRESPONSE
DESCRIPTOR.message_types_by_name['GetGridFaceEdgesResponse'] = _GETGRIDFACEEDGESRESPONSE
DESCRIPTOR.message_types_by_name['GetGridFaceNodesResponse'] = _GETGRIDFACENODESRESPONSE
DESCRIPTOR.message_types_by_name['GetGridNodesPerFaceResponse'] = _GETGRIDNODESPERFACERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.Empty)
  })
_sym_db.RegisterMessage(Empty)

InitializeRequest = _reflection.GeneratedProtocolMessageType('InitializeRequest', (_message.Message,), {
  'DESCRIPTOR' : _INITIALIZEREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.InitializeRequest)
  })
_sym_db.RegisterMessage(InitializeRequest)

GetComponentNameResponse = _reflection.GeneratedProtocolMessageType('GetComponentNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCOMPONENTNAMERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetComponentNameResponse)
  })
_sym_db.RegisterMessage(GetComponentNameResponse)

GetVarNamesResponse = _reflection.GeneratedProtocolMessageType('GetVarNamesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARNAMESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarNamesResponse)
  })
_sym_db.RegisterMessage(GetVarNamesResponse)

GetTimeUnitsResponse = _reflection.GeneratedProtocolMessageType('GetTimeUnitsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMEUNITSRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetTimeUnitsResponse)
  })
_sym_db.RegisterMessage(GetTimeUnitsResponse)

GetTimeStepResponse = _reflection.GeneratedProtocolMessageType('GetTimeStepResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMESTEPRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetTimeStepResponse)
  })
_sym_db.RegisterMessage(GetTimeStepResponse)

GetTimeResponse = _reflection.GeneratedProtocolMessageType('GetTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetTimeResponse)
  })
_sym_db.RegisterMessage(GetTimeResponse)

GetVarRequest = _reflection.GeneratedProtocolMessageType('GetVarRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETVARREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarRequest)
  })
_sym_db.RegisterMessage(GetVarRequest)

GetVarGridResponse = _reflection.GeneratedProtocolMessageType('GetVarGridResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARGRIDRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarGridResponse)
  })
_sym_db.RegisterMessage(GetVarGridResponse)

GetVarTypeResponse = _reflection.GeneratedProtocolMessageType('GetVarTypeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARTYPERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarTypeResponse)
  })
_sym_db.RegisterMessage(GetVarTypeResponse)

GetVarItemSizeResponse = _reflection.GeneratedProtocolMessageType('GetVarItemSizeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARITEMSIZERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarItemSizeResponse)
  })
_sym_db.RegisterMessage(GetVarItemSizeResponse)

GetVarUnitsResponse = _reflection.GeneratedProtocolMessageType('GetVarUnitsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARUNITSRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarUnitsResponse)
  })
_sym_db.RegisterMessage(GetVarUnitsResponse)

GetVarNBytesResponse = _reflection.GeneratedProtocolMessageType('GetVarNBytesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARNBYTESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarNBytesResponse)
  })
_sym_db.RegisterMessage(GetVarNBytesResponse)

GetVarLocationResponse = _reflection.GeneratedProtocolMessageType('GetVarLocationResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVARLOCATIONRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetVarLocationResponse)
  })
_sym_db.RegisterMessage(GetVarLocationResponse)

IntArrayMessage = _reflection.GeneratedProtocolMessageType('IntArrayMessage', (_message.Message,), {
  'DESCRIPTOR' : _INTARRAYMESSAGE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.IntArrayMessage)
  })
_sym_db.RegisterMessage(IntArrayMessage)

FloatArrayMessage = _reflection.GeneratedProtocolMessageType('FloatArrayMessage', (_message.Message,), {
  'DESCRIPTOR' : _FLOATARRAYMESSAGE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.FloatArrayMessage)
  })
_sym_db.RegisterMessage(FloatArrayMessage)

DoubleArrayMessage = _reflection.GeneratedProtocolMessageType('DoubleArrayMessage', (_message.Message,), {
  'DESCRIPTOR' : _DOUBLEARRAYMESSAGE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.DoubleArrayMessage)
  })
_sym_db.RegisterMessage(DoubleArrayMessage)

GetValueResponse = _reflection.GeneratedProtocolMessageType('GetValueResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVALUERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetValueResponse)
  })
_sym_db.RegisterMessage(GetValueResponse)

GetValueAtIndicesRequest = _reflection.GeneratedProtocolMessageType('GetValueAtIndicesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETVALUEATINDICESREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetValueAtIndicesRequest)
  })
_sym_db.RegisterMessage(GetValueAtIndicesRequest)

GetValueAtIndicesResponse = _reflection.GeneratedProtocolMessageType('GetValueAtIndicesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVALUEATINDICESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetValueAtIndicesResponse)
  })
_sym_db.RegisterMessage(GetValueAtIndicesResponse)

SetValueRequest = _reflection.GeneratedProtocolMessageType('SetValueRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETVALUEREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.SetValueRequest)
  })
_sym_db.RegisterMessage(SetValueRequest)

SetValuePtrRequest = _reflection.GeneratedProtocolMessageType('SetValuePtrRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETVALUEPTRREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.SetValuePtrRequest)
  })
_sym_db.RegisterMessage(SetValuePtrRequest)

SetValueAtIndicesRequest = _reflection.GeneratedProtocolMessageType('SetValueAtIndicesRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETVALUEATINDICESREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.SetValueAtIndicesRequest)
  })
_sym_db.RegisterMessage(SetValueAtIndicesRequest)

GridRequest = _reflection.GeneratedProtocolMessageType('GridRequest', (_message.Message,), {
  'DESCRIPTOR' : _GRIDREQUEST,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GridRequest)
  })
_sym_db.RegisterMessage(GridRequest)

GetGridSizeResponse = _reflection.GeneratedProtocolMessageType('GetGridSizeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDSIZERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridSizeResponse)
  })
_sym_db.RegisterMessage(GetGridSizeResponse)

GetGridRankResponse = _reflection.GeneratedProtocolMessageType('GetGridRankResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDRANKRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridRankResponse)
  })
_sym_db.RegisterMessage(GetGridRankResponse)

GetGridTypeResponse = _reflection.GeneratedProtocolMessageType('GetGridTypeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDTYPERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridTypeResponse)
  })
_sym_db.RegisterMessage(GetGridTypeResponse)

GetGridShapeResponse = _reflection.GeneratedProtocolMessageType('GetGridShapeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDSHAPERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridShapeResponse)
  })
_sym_db.RegisterMessage(GetGridShapeResponse)

GetGridSpacingResponse = _reflection.GeneratedProtocolMessageType('GetGridSpacingResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDSPACINGRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridSpacingResponse)
  })
_sym_db.RegisterMessage(GetGridSpacingResponse)

GetGridOriginResponse = _reflection.GeneratedProtocolMessageType('GetGridOriginResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDORIGINRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridOriginResponse)
  })
_sym_db.RegisterMessage(GetGridOriginResponse)

GetGridPointsResponse = _reflection.GeneratedProtocolMessageType('GetGridPointsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDPOINTSRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridPointsResponse)
  })
_sym_db.RegisterMessage(GetGridPointsResponse)

GetCountResponse = _reflection.GeneratedProtocolMessageType('GetCountResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCOUNTRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetCountResponse)
  })
_sym_db.RegisterMessage(GetCountResponse)

GetGridEdgeNodesResponse = _reflection.GeneratedProtocolMessageType('GetGridEdgeNodesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDEDGENODESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridEdgeNodesResponse)
  })
_sym_db.RegisterMessage(GetGridEdgeNodesResponse)

GetGridFaceEdgesResponse = _reflection.GeneratedProtocolMessageType('GetGridFaceEdgesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDFACEEDGESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridFaceEdgesResponse)
  })
_sym_db.RegisterMessage(GetGridFaceEdgesResponse)

GetGridFaceNodesResponse = _reflection.GeneratedProtocolMessageType('GetGridFaceNodesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDFACENODESRESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridFaceNodesResponse)
  })
_sym_db.RegisterMessage(GetGridFaceNodesResponse)

GetGridNodesPerFaceResponse = _reflection.GeneratedProtocolMessageType('GetGridNodesPerFaceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGRIDNODESPERFACERESPONSE,
  '__module__' : 'grpc4bmi.bmi_pb2'
  # @@protoc_insertion_point(class_scope:bmi.GetGridNodesPerFaceResponse)
  })
_sym_db.RegisterMessage(GetGridNodesPerFaceResponse)


_INTARRAYMESSAGE.fields_by_name['values']._options = None
_FLOATARRAYMESSAGE.fields_by_name['values']._options = None
_DOUBLEARRAYMESSAGE.fields_by_name['values']._options = None
_GETVALUEATINDICESREQUEST.fields_by_name['indices']._options = None
_SETVALUEATINDICESREQUEST.fields_by_name['indices']._options = None
_GETGRIDSHAPERESPONSE.fields_by_name['shape']._options = None
_GETGRIDSPACINGRESPONSE.fields_by_name['spacing']._options = None
_GETGRIDORIGINRESPONSE.fields_by_name['origin']._options = None
_GETGRIDPOINTSRESPONSE.fields_by_name['coordinates']._options = None
_GETGRIDEDGENODESRESPONSE.fields_by_name['edge_nodes']._options = None
_GETGRIDFACEEDGESRESPONSE.fields_by_name['face_edges']._options = None
_GETGRIDFACENODESRESPONSE.fields_by_name['face_nodes']._options = None
_GETGRIDNODESPERFACERESPONSE.fields_by_name['nodes_per_face']._options = None

_BMISERVICE = _descriptor.ServiceDescriptor(
  name='BmiService',
  full_name='bmi.BmiService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2189,
  serialized_end=4631,
  methods=[
  _descriptor.MethodDescriptor(
    name='initialize',
    full_name='bmi.BmiService.initialize',
    index=0,
    containing_service=None,
    input_type=_INITIALIZEREQUEST,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='update',
    full_name='bmi.BmiService.update',
    index=1,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='updateUntil',
    full_name='bmi.BmiService.updateUntil',
    index=2,
    containing_service=None,
    input_type=_GETTIMERESPONSE,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='finalize',
    full_name='bmi.BmiService.finalize',
    index=3,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getComponentName',
    full_name='bmi.BmiService.getComponentName',
    index=4,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCOMPONENTNAMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInputItemCount',
    full_name='bmi.BmiService.getInputItemCount',
    index=5,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOutputItemCount',
    full_name='bmi.BmiService.getOutputItemCount',
    index=6,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInputVarNames',
    full_name='bmi.BmiService.getInputVarNames',
    index=7,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETVARNAMESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOutputVarNames',
    full_name='bmi.BmiService.getOutputVarNames',
    index=8,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETVARNAMESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTimeUnits',
    full_name='bmi.BmiService.getTimeUnits',
    index=9,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMEUNITSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTimeStep',
    full_name='bmi.BmiService.getTimeStep',
    index=10,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMESTEPRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getCurrentTime',
    full_name='bmi.BmiService.getCurrentTime',
    index=11,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getStartTime',
    full_name='bmi.BmiService.getStartTime',
    index=12,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getEndTime',
    full_name='bmi.BmiService.getEndTime',
    index=13,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarGrid',
    full_name='bmi.BmiService.getVarGrid',
    index=14,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARGRIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarType',
    full_name='bmi.BmiService.getVarType',
    index=15,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARTYPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarItemSize',
    full_name='bmi.BmiService.getVarItemSize',
    index=16,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARITEMSIZERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarUnits',
    full_name='bmi.BmiService.getVarUnits',
    index=17,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARUNITSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarNBytes',
    full_name='bmi.BmiService.getVarNBytes',
    index=18,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARNBYTESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getVarLocation',
    full_name='bmi.BmiService.getVarLocation',
    index=19,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVARLOCATIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getValue',
    full_name='bmi.BmiService.getValue',
    index=20,
    containing_service=None,
    input_type=_GETVARREQUEST,
    output_type=_GETVALUERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getValueAtIndices',
    full_name='bmi.BmiService.getValueAtIndices',
    index=21,
    containing_service=None,
    input_type=_GETVALUEATINDICESREQUEST,
    output_type=_GETVALUEATINDICESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setValue',
    full_name='bmi.BmiService.setValue',
    index=22,
    containing_service=None,
    input_type=_SETVALUEREQUEST,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setValueAtIndices',
    full_name='bmi.BmiService.setValueAtIndices',
    index=23,
    containing_service=None,
    input_type=_SETVALUEATINDICESREQUEST,
    output_type=_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridSize',
    full_name='bmi.BmiService.getGridSize',
    index=24,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDSIZERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridType',
    full_name='bmi.BmiService.getGridType',
    index=25,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDTYPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridRank',
    full_name='bmi.BmiService.getGridRank',
    index=26,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDRANKRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridShape',
    full_name='bmi.BmiService.getGridShape',
    index=27,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDSHAPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridSpacing',
    full_name='bmi.BmiService.getGridSpacing',
    index=28,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDSPACINGRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridOrigin',
    full_name='bmi.BmiService.getGridOrigin',
    index=29,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDORIGINRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridX',
    full_name='bmi.BmiService.getGridX',
    index=30,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDPOINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridY',
    full_name='bmi.BmiService.getGridY',
    index=31,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDPOINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridZ',
    full_name='bmi.BmiService.getGridZ',
    index=32,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDPOINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridNodeCount',
    full_name='bmi.BmiService.getGridNodeCount',
    index=33,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridEdgeCount',
    full_name='bmi.BmiService.getGridEdgeCount',
    index=34,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridFaceCount',
    full_name='bmi.BmiService.getGridFaceCount',
    index=35,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridEdgeNodes',
    full_name='bmi.BmiService.getGridEdgeNodes',
    index=36,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDEDGENODESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridFaceNodes',
    full_name='bmi.BmiService.getGridFaceNodes',
    index=37,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDFACENODESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridFaceEdges',
    full_name='bmi.BmiService.getGridFaceEdges',
    index=38,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDFACEEDGESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getGridNodesPerFace',
    full_name='bmi.BmiService.getGridNodesPerFace',
    index=39,
    containing_service=None,
    input_type=_GRIDREQUEST,
    output_type=_GETGRIDNODESPERFACERESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_BMISERVICE)

DESCRIPTOR.services_by_name['BmiService'] = _BMISERVICE

# @@protoc_insertion_point(module_scope)
